<?php

namespace FiloBlu\Refilo\Remote;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterfaceFactory;
use FiloBlu\Refilo\Remote\Helper\TemplateStringFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationRepositoryInterface;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerResolver;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class AbstractStoreBasedIndexer
 * @package FiloBlu\Refilo\Remote
 */
abstract class AbstractStoreBasedIndexer extends AbstractIndexer
{
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * AbstractStoreBasedIndexer constructor.
     * @param StoreManagerInterface $storeManager
     * @param EntityProviderInterfaceFactory $entityProviderFactory
     * @param IndexerConfigurationRepositoryInterface $indexerConfigurationRepository
     * @param TemplateStringFactory $templateStringFactory
     * @param IndexerResolver $indexerResolver
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        EntityProviderInterfaceFactory $entityProviderFactory,
        IndexerConfigurationRepositoryInterface $indexerConfigurationRepository,
        TemplateStringFactory $templateStringFactory,
        IndexerResolver $indexerResolver,
        ScopeConfigInterface $scopeConfig,
        LoggerInterface $logger
    ) {
        parent::__construct(
            $entityProviderFactory,
            $indexerConfigurationRepository,
            $templateStringFactory,
            $indexerResolver,
            $scopeConfig,
            $logger
        );
        $this->storeManager = $storeManager;
    }

    /**
     * @param array $ids
     * @return mixed|void
     * @throws NoSuchEntityException
     */
    public function executeAction(array $ids)
    {
        $indexTemplate = $this->getGetRemoteIndexTemplate();

        foreach ($this->storeManager->getStores() as $store) {
            if (!$store->getIsActive()) {
                continue;
            }

            $entities = $this->entityProviderFactory->create()
                ->forStore($store)
                ->withIds($ids);

            $collection = $indexTemplate->withValue('%s', $store->getCode())->render();
            $this->write($entities, $collection);
        }
    }
}
